using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

using System.Xml;
using System.Xml.Serialization;
using System.IO;
using System.Diagnostics;
using System.Net;

namespace AddressCheckXML
{
    public partial class AddressCheckXML : Form
    {
        public AddressCheckXML()
        {
            InitializeComponent();
        }


        // Global Variables
        HttpWebRequest request;
        Stream responseReader;
        Uri address;
        byte[] byteData;
        Stream postStream = null;
        String tServer;
        int recordCount = 0;
        int tempRecordCount = 1;
        int currentRecord = 1;
        XmlTextWriter Writer;
        StreamReader fp;
        String xmlData;
        XmlDocument xmlDoc;
        System.Xml.XmlDocument rootNode = new System.Xml.XmlDocument();
        Boolean isInitialized = true;



        //***************************************************************************
        // Function to Initialize DQWS V2 XML
        //****************************************************************************
        private void Initialize()
        {
            // Initialize XML Text Writer
            Writer = new XmlTextWriter("request.xml", null);

            // Write XML Header
            Writer.WriteStartDocument();
            Writer.Formatting = Formatting.Indented;
            Writer.Indentation = 3;

            Writer.WriteStartElement("RequestArray");
            Writer.WriteElementString("TransmissionReference", "urn:mdWebServiceAddress", "Testing: DQWS XML Sample Code implementation using multiple record inputs.");
            Writer.WriteElementString("CustomerID", txtCustomerID.Text);
            Writer.WriteElementString("OptAddressParsed", "True");


        }


        //********************************************************************************
        // Function to Begin Address Verification Process for Single and Multiple Records
        //*********************************************************************************
        private void btnLookUp_Click(object sender, EventArgs e)
        {
            // Clear the XML Each Time
            webBrowser1.Navigate("about:blank");

            // If recordCount = 0 that means there are no records, then simply do the current user inputs
            if (recordCount == 0)
            {
                // Initialize XML Text Writer
                Initialize();

                // Keep inserting records until btnLookup is pressed
                Writer.WriteStartElement("Record");
                Writer.WriteElementString("RecordID", Convert.ToString(recordCount + 1));
                Writer.WriteElementString("Company", txtCompany.Text);
                Writer.WriteElementString("Urbanization", txtUrbanization.Text);
                Writer.WriteElementString("AddressLine1", txtAddress1.Text);
                Writer.WriteElementString("AddressLine2", txtAddress2.Text);
                Writer.WriteElementString("Suite", txtSuite.Text);
                Writer.WriteElementString("City", txtCity.Text);
                Writer.WriteElementString("State", txtState.Text);
                Writer.WriteElementString("Zip", txtZip.Text);
                Writer.WriteElementString("Plus4", txtPlus4.Text);
                Writer.WriteElementString("Country", txtCountry.Text);
                Writer.WriteElementString("LastName", txtLastName.Text);

                Writer.WriteEndElement();

                // Increment Record to 1
                recordCount = 1;
            }


            // When the lookup button is pressed, then we can end XmlWriter
            Writer.WriteEndElement();
            Writer.Close();

            // Set the Server URL to the form input
            tServer = "https://addresscheck.melissadata.net/v2/XML/Service.svc/doAddressCheck";
            address = new Uri(tServer);

            // Create the web request 
            try
            {
                request = (HttpWebRequest)WebRequest.Create(address);
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.ToString());
            }


            request.Method = "POST";
            request.ContentType = "application/xml; charset=utf-8";

            // Read in XML file and then convert it to string
            fp = File.OpenText("request.xml");
            xmlData = fp.ReadToEnd();
            fp.Close();

            byteData = UTF8Encoding.UTF8.GetBytes(xmlData.ToString());

            // Set the content length in the request headers   
            request.ContentLength = byteData.Length;

            // Write data to a stream
            try
            {
                postStream = request.GetRequestStream();
                postStream.Write(byteData, 0, byteData.Length);
            }
            finally
            {
                if (postStream != null)
                {
                    postStream.Close();
                }
            }

            // Get response   
            HttpWebResponse response = (HttpWebResponse)request.GetResponse();

            // Get the response stream into a reader   
            responseReader = response.GetResponseStream();

            XmlTextWriter outXML;
            outXML = new XmlTextWriter("response.xml", null);

            xmlDoc = new XmlDocument();
            xmlDoc.Load(responseReader);
            xmlDoc.Save(outXML);
            Writer.Close();
            outXML.Close();

            rootNode.Load("Response.xml");

            // Clear previous results, then re-populate fields.
            ClearResults();

            // Fill the Individual Fields. Each time verify is hit, must return record counter to 1
            // Must be initialized correctly to continue
            currentRecord = 1;
            isInitialized = true;
            PopulateNodes(rootNode);

            if (isInitialized)
            {
                btnNext.Enabled = true;
                btnPrevious.Enabled = true;

                // View response xml
                webBrowser1.Navigate(System.Environment.CurrentDirectory + "\\response.xml");

                responseReader.Close();

                // Set records back to 0. Everytime the records are processed, XML requests must be redone.
                tempRecordCount = recordCount;
                txtRecNo.Text = currentRecord.ToString();
                txtTotal.Text = recordCount.ToString();

                txtRecordCount.Text = recordCount.ToString();
            }
            else
            {
                btnNext.Enabled = false;
                btnPrevious.Enabled = false;

                tempRecordCount = 1;
                currentRecord = 1;

                txtTotal.Text = Convert.ToString(0);
                txtRecNo.Text = Convert.ToString(0);
            }

            // Reset Record Counts
            recordCount = 0;
            txtRecordCount.Text = recordCount.ToString();
        }

        private void btnInsert_Click(object sender, EventArgs e)
        {
            //***************************************************************************
            // Function to Insert a Record for Multiple Record Handling
            //****************************************************************************

            // Execute this only if no records have been inserted yet.
            if (recordCount == 0)
            {
                //Initialize XML Text Writer
                Initialize();
            }

            // Keep inserting records until btnLookup is pressed
            Writer.WriteStartElement("Record");
            Writer.WriteElementString("RecordID", Convert.ToString(recordCount + 1));
            Writer.WriteElementString("Company", txtCompany.Text);
            Writer.WriteElementString("Urbanization", txtUrbanization.Text);
            Writer.WriteElementString("AddressLine1", txtAddress1.Text);
            Writer.WriteElementString("AddressLine2", txtAddress2.Text);
            Writer.WriteElementString("Suite", txtSuite.Text);
            Writer.WriteElementString("City", txtCity.Text);
            Writer.WriteElementString("State", txtState.Text);
            Writer.WriteElementString("Zip", txtZip.Text);
            Writer.WriteElementString("Plus4", txtPlus4.Text);
            Writer.WriteElementString("Country", txtCountry.Text);
            Writer.WriteElementString("LastName", txtLastName.Text);
            Writer.WriteEndElement();

            // Increment Record Count
            recordCount = recordCount + 1;
            txtRecordCount.Text = recordCount.ToString();
        }

        private void btnClear_Click(object sender, EventArgs e)
        {
            // Clear AddressCheck Fields
            txtAddress1.Text = "";
            txtAddress2.Text = "";
            txtCity.Text = "";
            txtPlus4.Text = "";
            txtZip.Text = "";
            txtState.Text = "";
            txtCompany.Text = "";
            txtSuite.Text = "";
            txtUrbanization.Text = "";
            txtCountry.Text = "";
            txtLastName.Text = "";
        }

        //***************************************************************************
        // Function to Clear Output Fields
        //****************************************************************************
        private void ClearResults()
        {
            cAddress1.Clear();
            cAddress2.Clear();
            cCompany.Clear();
            cCity.Clear();
            cState.Clear();
            cZip.Clear();
            cPlus4.Clear();
            cCountry.Clear();
            cRange.Clear();
            cStreetName.Clear();
            cSuffix.Clear();
            cPreDirection.Clear();
            cPostDirection.Clear();
            cSuiteName.Clear();
            cSuiteNumber.Clear();
            cPMBName.Clear();
            cPMBNumber.Clear();
            cUCode.Clear();
            cUName.Clear();
            cDPC.Clear();
            cDPCD.Clear();
            cCarrierRoute.Clear();
            cResultCode.Clear();
            cResultDescription.Clear();
            cInitErrCde.Clear();
            cInitErrDesc.Clear();
            cAddKey.Clear();
            cCongressDistrict.Clear();
        }


        // Populate the Node List through a Recursive Call from the root
        public void PopulateNodes(XmlDocument xml)
        {
            foreach (XmlNode xmlnode in xml)
            {
                SearchRecord(xmlnode);
            }
        }

        // Search for Current Record In the Record List
        public void SearchRecord(XmlNode xml)
        {
            //For Each parentNode As XmlNode In xml.ChildNodes
            foreach (XmlNode xmlNode in xml)
            {

                if ((xmlNode.Name == "Results") && (xmlNode.HasChildNodes))
                {
                    // ***********************************************************************
                    // Block to Handle Different Error Codes in Initialization Results
                    String errorCodeString = xmlNode.FirstChild.Value;
                    String[] errorCodeList = null;

                    errorCodeList = errorCodeString.Split(',');

                    foreach (String errorCode in errorCodeList)
                    {

                        cInitErrCde.Text += errorCode + "; ";

                        if (errorCode == "SE01")
                        {
                            cInitErrDesc.Text += errorCode + ": Web Service Internal Error;  ";
                        }
                        else if (errorCode == "GE01")
                        {
                            cInitErrDesc.Text += errorCode + ": Empty XML Request Structure;  ";
                        }
                        else if (errorCode == "GE02")
                        {
                            cInitErrDesc.Text += errorCode + ": Empty XML Request Record Structure;  ";
                        }
                        else if (errorCode == "GE03")
                        {
                            cInitErrDesc.Text += errorCode + ": Counted records send more than number of records allowed per request;  ";
                        }
                        else if (errorCode == "GE04")
                        {
                            cInitErrDesc.Text += errorCode + ": CustomerID empty;  ";
                            MessageBox.Show("Please enter your Customer ID", "Invalid Customer ID");
                        }
                        else if (errorCode == "GE05")
                        {
                            cInitErrDesc.Text += errorCode + ": CustomerID not valid;  ";
                            MessageBox.Show("The Customer ID your entered is invalid. To retrieve a valid " +
                                 "Customer ID, please contact a Melissa Data Sales Representative " +
                                 "at 800-MELISSA ext. 3 (800-800-6245 ext. 3). ", "Invalid Customer ID");
                        }
                        else if (errorCode == "GE06")
                        {
                            cInitErrDesc.Text += errorCode + ": CustomerID disabled;  ";
                            MessageBox.Show("The Customer ID your entered has been disabled. " +
                                 "Please contact a Melissa Data Sales Representative " +
                                 "at 800-MELISSA ext. 3 (800-800-6245 ext. 3). ", "Disabled Customer ID");
                        }
                        else if (errorCode == "GE07")
                        {
                            cInitErrDesc.Text += errorCode + ": XML Request invalid;  ";
                        }
                    }

                    // Did not Initialize. Disable Buttons
                    isInitialized = false;
                    return;
                }
                //***********************************************************************

                else if ((xmlNode.Name == "Results") && !(xmlNode.HasChildNodes))
                {
                    // If No Initialize Error
                    cInitErrDesc.Text = "No Error";
                }

                else if (xmlNode.Name == "Record")
                {
                    // If The RecordID is equal to the current node being selected, then output all it's sub nodes.
                    if ((xmlNode.FirstChild.Name == "RecordID") && (xmlNode.FirstChild.FirstChild.Value == currentRecord.ToString()))
                    {
                        PopulateSubNodes(xmlNode);
                    }
                }
            }
        }


        // Popoulate all Sub Nodes below the chosen Record Node
        public void PopulateSubNodes(XmlNode parentxmlnode)
        {
            foreach (XmlNode node in parentxmlnode)
            {
                Add(node);
                PopulateSubNodes(node);
            }
        }


        //***************************************************************************
        // Functions to Populate The Output Fields
        //****************************************************************************
        // Populate Output Fields that have existing nodes
        public void Add(XmlNode theNode)
        {

            if (theNode.ParentNode.Name == "Results")
            {

                //**********************************************************************************
                // Block to Handle Different Error Codes and Status Codes in each Record
                //**********************************************************************************
                String errorCodeString = theNode.Value;
                String[] errorCodeList = null;

                errorCodeList = errorCodeString.Split(',');

                foreach (String errorCode in errorCodeList)
                {

                    cResultCode.Text += errorCode + "; ";

                    if (errorCode == "AE01")
                        cResultDescription.Text += errorCode + ": Zip Code Error;  ";
                    else if (errorCode == "AE02")
                        cResultDescription.Text += errorCode + ": Unknown Street;  ";
                    else if (errorCode == "AE03")
                        cResultDescription.Text += errorCode + ": Component Error;  ";
                    else if (errorCode == "AE04")
                        cResultDescription.Text += errorCode + ": Non-Deliverable Address;  ";
                    else if (errorCode == "AE05")
                        cResultDescription.Text += errorCode + ": Address Matched to Multiple Records;  ";
                    else if (errorCode == "AE06")
                        cResultDescription.Text += errorCode + ": Address Matched to Early Warning System;  ";
                    else if (errorCode == "AE07")
                        cResultDescription.Text += errorCode + ": Empty Address Input;  ";
                    else if (errorCode == "AE08")
                        cResultDescription.Text += errorCode + ": Suite Range Error;  ";
                    else if (errorCode == "AE09")
                        cResultDescription.Text += errorCode + ": Suite Range Missing;  ";
                    else if (errorCode == "AE10")
                        cResultDescription.Text += errorCode + ": Primary Range Error;  ";
                    else if (errorCode == "AE11")
                        cResultDescription.Text += errorCode + ": Primary Range Missing;  ";
                    else if (errorCode == "AE12")
                        cResultDescription.Text += errorCode + ": Box Number Error from PO Box or RR;  ";
                    else if (errorCode == "AE13")
                        cResultDescription.Text += errorCode + ": PO Box Number Missing from PO Box or RR;  ";
                    else if (errorCode == "AE14")
                        cResultDescription.Text += errorCode + ": Input Address Matched to CMRA but secondary number not present;  ";
                    else if (errorCode == "AE17")
                        cResultDescription.Text += errorCode + ": A suite number was entered but no suite information found for primary address;  ";

                    else if (errorCode == "AS01")
                        cResultDescription.Text += errorCode + ": Address matched to postal database;  ";
                    else if (errorCode == "AS02")
                        cResultDescription.Text += errorCode + ": Address matched to non-postal database;  ";
                    else if (errorCode == "AS09")
                        cResultDescription.Text += errorCode + ": Foreign Postal Code Detected;  ";
                    else if (errorCode == "AS10")
                        cResultDescription.Text += errorCode + ": Address matched to CMRA;  ";
                    else if (errorCode == "AS13")
                        cResultDescription.Text += errorCode + ": Address Updated By LACS;  ";
                    else if (errorCode == "AS14")
                        cResultDescription.Text += errorCode + ": Address Updated By Suite Link;  ";
                    else if (errorCode == "AS15")
                        cResultDescription.Text += errorCode + ": Address Updated By AddressPlus;  ";
                    else if (errorCode == "AS16")
                        cResultDescription.Text += errorCode + ": Address is vacant;  ";
                    else if (errorCode == "AS17")
                        cResultDescription.Text += errorCode + ": Alternate delivery;  ";
                    else if (errorCode == "AS18")
                        cResultDescription.Text += errorCode + ": Artificially created adresses detected,DPV processing terminated at this point;  ";
                    else if (errorCode == "AS20")
                        cResultDescription.Text += errorCode + ": Address Deliverable by USPS only;  ";
                    else if (errorCode == "AS23")
                        cResultDescription.Text += errorCode + ": Extraneous information foundr;  ";

                    else if (errorCode == "AC01")
                        cResultDescription.Text += errorCode + ": ZIP Code Change;  ";
                    else if (errorCode == "AC02")
                        cResultDescription.Text += errorCode + ": State Change;  ";
                    else if (errorCode == "AC03")
                        cResultDescription.Text += errorCode + ": City Change;  ";
                    else if (errorCode == "AC04")
                        cResultDescription.Text += errorCode + ": Base/Alternate Changed;  ";
                    else if (errorCode == "AC05")
                        cResultDescription.Text += errorCode + ": Alias Name Change;  ";
                    else if (errorCode == "AC06")
                        cResultDescription.Text += errorCode + ": Address1/Address2 Swap;  ";
                    else if (errorCode == "AC07")
                        cResultDescription.Text += errorCode + ": Address1/Company Swap;  ";
                    else if (errorCode == "AC08")
                        cResultDescription.Text += errorCode + ": Plus4 Change;  ";
                    else if (errorCode == "AC09")
                        cResultDescription.Text += errorCode + ": Urbanization Change;  ";
                    else if (errorCode == "AC10")
                        cResultDescription.Text += errorCode + ": Street Name Change;  ";
                    else if (errorCode == "AC11")
                        cResultDescription.Text += errorCode + ": Street Suffix Change;  ";
                    else if (errorCode == "AC12")
                        cResultDescription.Text += errorCode + ": Street Directional Change;  ";
                    else if (errorCode == "AC13")
                        cResultDescription.Text += errorCode + ": Suite Name Change;  ";

                        

                }

                if (errorCodeString == "AS01,AS12")
                {
                    cResultDescription.Clear();
                    cResultDescription.Text += "Input Address Matched to DPV;  "; ;
                }
                // End of Error Handling
                //*********************************************************************************
            }

            else if (theNode.ParentNode.Name == "Company")
                cCompany.Text = theNode.Value;
            else if (theNode.ParentNode.Name == "Address1")
                cAddress1.Text = theNode.Value;
            else if (theNode.ParentNode.Name == "Address2")
                cAddress2.Text = theNode.Value;
            else if (theNode.ParentNode.Name == "Zip")
            {
                if (theNode.ParentNode.ParentNode.Name == "Address")
                    cZip.Text = theNode.Value;
            }
            else if (theNode.ParentNode.Name == "Plus4")
                cPlus4.Text = theNode.Value;
            else if (theNode.ParentNode.Name == "AddressRange")
                cRange.Text = theNode.Value;
            else if (theNode.ParentNode.Name == "StreetName")
                cStreetName.Text = theNode.Value;
            else if (theNode.ParentNode.Name == "Suffix")
                cSuffix.Text = theNode.Value;
            else if (theNode.ParentNode.Name == "Pre")
                cPreDirection.Text = theNode.Value;
            else if (theNode.ParentNode.Name == "Post")
                cPostDirection.Text = theNode.Value;
            else if (theNode.ParentNode.Name == "AddressKey")
                cAddKey.Text = theNode.Value;
            else if (theNode.ParentNode.Name == "DeliveryPointCode")
                cDPC.Text = theNode.Value;
            else if (theNode.ParentNode.Name == "DeliveryPointCheckDigit")
                cDPCD.Text = theNode.Value;
            else if (theNode.ParentNode.Name == "CarrierRoute")
                cCarrierRoute.Text = theNode.Value;
            else if (theNode.ParentNode.Name == "CongressionalDistrict")
                cCongressDistrict.Text = theNode.Value;

            else if (theNode.ParentNode.Name == "Abbreviation")
            {
                // Several Nodes use "Code". We must handle and distinguish which Code is refering to
                if (theNode.ParentNode.ParentNode.Name == "State")
                    cState.Text = theNode.Value;
            }
            else if (theNode.ParentNode.Name == "Code")
            {
                // Several Nodes use "Code". We must handle and distinguish which Code is refering to
                if (theNode.ParentNode.ParentNode.Name == "Urbanization")
                    cUCode.Text = theNode.Value;
            }
            else if (theNode.ParentNode.Name == "Range")
            {
                // Several Nodes use "Range". We must handle and distinguish which Range is refering to
                if (theNode.ParentNode.ParentNode.Name == "Suite")
                    cSuiteNumber.Text = theNode.Value;
                else if (theNode.ParentNode.ParentNode.Name == "PrivateMailbox")
                    cPMBNumber.Text = theNode.Value;
            }
            else if (theNode.ParentNode.Name == "Name")
            {
                // Several Nodes use "Name". We must handle and distinguish which name is refering to
                if (theNode.ParentNode.ParentNode.Name == "Suite")
                    cSuiteName.Text = theNode.Value;
                else if (theNode.ParentNode.ParentNode.Name == "City")
                    cCity.Text = theNode.Value;
                else if (theNode.ParentNode.ParentNode.Name == "Urbanization")
                    cUName.Text = theNode.Value;
                else if (theNode.ParentNode.ParentNode.Name == "Country")
                    cCountry.Text = theNode.Value;
                else if (theNode.ParentNode.ParentNode.Name == "PrivateMailbox")
                    cPMBName.Text = theNode.Value;
            }

        }

        private void btnPrevious_Click(object sender, EventArgs e)
        {
            // Clear Fields Each Time
            ClearResults();

            // Decrement the Record Counter
            if (currentRecord != 1)
                currentRecord = currentRecord - 1;
            else
                currentRecord = tempRecordCount;

            // Fill the Individual Fields
            PopulateNodes(rootNode);

            // Update Record Counter
            txtRecNo.Text = currentRecord.ToString();
        }

        private void btnNext_Click(object sender, EventArgs e)
        {
            // Clear Fields Each Time
            ClearResults();

            // Increment the Record Counter
            if (currentRecord != tempRecordCount)
                currentRecord = currentRecord + 1;
            else
                currentRecord = 1;

            // Fill the Individual Fields
            PopulateNodes(rootNode);

            // Update Record Counter
            txtRecNo.Text = currentRecord.ToString();
        }
    }
}